bin='/SDK/Local/C'
binexp='/SDK/Local/C'
ccflags='-fno-strict-aliasing  -mcrt=newlib -I/SDK/newlib/include -I/SDK/local/newlib/include -I/SDK/Local/common/include -DDEBUGGING'
cppflags='-fno-strict-aliasing -I/SDK/newlib/include -I/SDK/local/newlib/include -I/SDK/Local/common/include -DDEBUGGING'
initialinstalllocation='/SDK/Local/C'
installbin='/SDK/Local/C'
installprefix='/SDK/Local/newlib'
installprefixexp='/SDK/Local/newlib'
installscript='/SDK/Local/C'
installsitebin='/SDK/Local/C'
installsitescript='/SDK/Local/C'
libc='/SDK/newlib/lib/libc.a'
libpth='/SDK/newlib/lib /SDK/local/newlib/lib'
libsdirs=' /SDK/newlib/lib'
libsfound=' /SDK/newlib/lib/libsocket.a /SDK/newlib/lib/libm.a /SDK/newlib/lib/libc.a'
libspath=' /SDK/newlib/lib /SDK/local/newlib/lib'
make_set_make='MAKE=/SDK/C/gmake'
pager='/SYS/Utilities/MultiView'
perl5='/SDK/Local/C/perl'
perlpath='/SDK/Local/C/perl'
prefix='/SDK/Local/newlib'
prefixexp='/SDK/Local/newlib'
scriptdir='/SDK/Local/C'
scriptdirexp='/SDK/Local/C'
sitebin='/SDK/Local/C'
sitebinexp='/SDK/Local/C'
siteprefix='/SDK/Local'
siteprefixexp='/SDK/Local'
sitescript='/SDK/Local/C'
sitescriptexp='/SDK/Local/C'
startperl='#!SDK:Local/C/perl'
startsh='#!SDK:C/sh'
xlibpth='/lib /SDK/newlib/lib'
lddlflags='-shared'
ldflags='-mcrt=newlib'
ldflags_uselargefiles=''
dlext='pso'
cccdlflags='-fPIC'
ccdlflags='-use-dynld  -Wl,--export-dynamic'
libs='-lm -lauto -lpthread -lunix'
usedl='define'
useithreads='define'
usemultiplicity='define'
usemymalloc='n'
usenm='false'
usethreads='define'
dlsrc='dl_dlopen.xs'
d_procselfexe='undef'
sig_count='50'
sig_name='ZERO HUP INT QUIT ILL TRAP ABRT EMT FPE KILL BUS SEGV SYS PIPE ALRM TERM URG STOP TSTP CONT CHLD TTIN TTOU IO XCPU XFSZ VTALRM PROF WINCH LOST USR1 USR2 NUM32 NUM33 NUM34 NUM35 NUM36 NUM37 NUM38 NUM39 NUM40 NUM41 NUM42 NUM43 NUM44 NUM45 NUM46 NUM47 NUM48 NUM49 CLD IOT POLL '
sig_name_init='"ZERO", "HUP", "INT", "QUIT", "ILL", "TRAP", "ABRT" , "EMT", "FPE",  "KILL", "BUS", "SEGV","SYS","PIPE","ALRM","TERM","URG","STOP","TSTP","CONT", "CHLD","TTIN", "TTOU", "IO" , "XCPU", "XFSZ", "VTALRM", "PROF", "WINCH", "LOST" , "USR1", "USR2", "NUM32", "NUM33", "NUM34", "NUM35", "NUM36", "NUM37", "NUM38", "NUM39", "NUM40", "NUM41", "NUM42", "NUM43", "NUM44", "NUM45", "NUM46", "NUM47", "NUM48", "NUM49", "CLD", "IOT" , "POLL"'
sig_num='0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 20 6 23 '
sig_num_init='0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 20, 6, 23 '
sig_size='53'
optimize='-gstabs -O2'
archobjs='amigaos.o amigaio.o'
firstmakefile='gnumakefile'
i_sysfile='undef' # just a wrapper for fnctl and gets included after
i_fcntl='define'
h_fcntl='true'
h_sysfile='false'
i_syswait='undef'
d_flock='define'
d_flockproto='define'
