$EPERM = 0x1;
$ENOENT = 0x2;
$ESRCH = 0x3;
$EINTR = 0x4;
$EIO = 0x5;
$ENXIO = 0x6;
$E2BIG = 0x7;
$ENOEXEC = 0x8;
$EBADF = 0x9;
$ECHILD = 0xA;
$EAGAIN = 0xB;
$ENOMEM = 0xC;
$EACCES = 0xD;
$EFAULT = 0xE;
$ENOTBLK = 0xF;
$EBUSY = 0x10;
$EEXIST = 0x11;
$EXDEV = 0x12;
$ENODEV = 0x13;
$ENOTDIR = 0x14;
$EISDIR = 0x15;
$EINVAL = 0x16;
$ENFILE = 0x17;
$EMFILE = 0x18;
$ENOTTY = 0x19;
$ETXTBSY = 0x1A;
$EFBIG = 0x1B;
$ENOSPC = 0x1C;
$ESPIPE = 0x1D;
$EROFS = 0x1E;
$EMLINK = 0x1F;
$EPIPE = 0x20;
$EDOM = 0x21;
$ERANGE = 0x22;
$EWOULDBLOCK = 0x23;
$EINPROGRESS = 0x24;
$EALREADY = 0x25;
$ENOTSOCK = 0x26;
$EDESTADDRREQ = 0x27;
$EMSGSIZE = 0x28;
$EPROTOTYPE = 0x29;
$ENOPROTOOPT = 0x2A;
$EPROTONOSUPPORT = 0x2B;
$ESOCKTNOSUPPORT = 0x2C;
$EOPNOTSUPP = 0x2D;
$EPFNOSUPPORT = 0x2E;
$EAFNOSUPPORT = 0x2F;
$EADDRINUSE = 0x30;
$EADDRNOTAVAIL = 0x31;
$ENETDOWN = 0x32;
$ENETUNREACH = 0x33;
$ENETRESET = 0x34;
$ECONNABORTED = 0x35;
$ECONNRESET = 0x36;
$ENOBUFS = 0x37;
$EISCONN = 0x38;
$ENOTCONN = 0x39;
$ESHUTDOWN = 0x3A;
$ETOOMANYREFS = 0x3B;
$ETIMEDOUT = 0x3C;
$ECONNREFUSED = 0x3D;
$ELOOP = 0x3E;
$ENAMETOOLONG = 0x3F;
$EHOSTDOWN = 0x40;
$EHOSTUNREACH = 0x41;
$ENOTEMPTY = 0x42;
$EPROCLIM = 0x43;
$EUSERS = 0x44;
$EDQUOT = 0x45;
$ESTALE = 0x46;
$EREMOTE = 0x47;
$EDEADLK = 0x48;
$ENOLCK = 0x49;
$MTH_UNDEF_SQRT = 0x12C;
$MTH_OVF_EXP = 0x12D;
$MTH_UNDEF_LOG = 0x12E;
$MTH_NEG_BASE = 0x12F;
$MTH_ZERO_BASE = 0x130;
$MTH_OVF_POW = 0x131;
$MTH_LRG_SIN = 0x132;
$MTH_LRG_COS = 0x133;
$MTH_LRG_TAN = 0x134;
$MTH_LRG_COT = 0x135;
$MTH_OVF_TAN = 0x136;
$MTH_OVF_COT = 0x137;
$MTH_UNDEF_ASIN = 0x138;
$MTH_UNDEF_ACOS = 0x139;
$MTH_UNDEF_ATAN2 = 0x13A;
$MTH_OVF_SINH = 0x13B;
$MTH_OVF_COSH = 0x13C;
$MTH_UNDEF_ZLOG = 0x13D;
$MTH_UNDEF_ZDIV = 0x13E;
