+{
   locale_version => 1.29,
   entry => <<'ENTRY', # for DUCET v13.0.0
0451      ; [.23C0.0020.0002] # CYRILLIC SMALL LETTER IO
0435 0308 ; [.23C0.0020.0002] # CYRILLIC SMALL LETTER IO
0401      ; [.23C0.0020.0008] # CYRILLIC CAPITAL LETTER IO
0415 0308 ; [.23C0.0020.0008] # CYRILLIC CAPITAL LETTER IO
04AF      ; [.248B.0020.0002] # CYRILLIC SMALL LETTER STRAIGHT U
04AE      ; [.248B.0020.0008] # CYRILLIC CAPITAL LETTER STRAIGHT U
0456      ; [.24FC.0020.0002] # CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I
0406      ; [.24FC.0020.0008] # CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I
0457      ; [.24FC.0020.0002][.0000.002B.0002] # CYRILLIC SMALL LETTER YI
A676      ; [.24FC.0020.0004][.0000.002B.0004] # COMBINING CYRILLIC LETTER YI
0407      ; [.24FC.0020.0008][.0000.002B.0002] # CYRILLIC CAPITAL LETTER YI
ENTRY
};
