+{
   locale_version => 1.29,
   entry => <<'ENTRY', # for DUCET v13.0.0
010D      ; [.1FEA.0020.0002] # LATIN SMALL LETTER C WITH CARON
0063 030C ; [.1FEA.0020.0002] # LATIN SMALL LETTER C WITH CARON
010C      ; [.1FEA.0020.0008] # LATIN CAPITAL LETTER C WITH CARON
0043 030C ; [.1FEA.0020.0008] # LATIN CAPITAL LETTER C WITH CARON
0123      ; [.2074.0020.0002] # LATIN SMALL LETTER G WITH CEDILLA
0067 0327 ; [.2074.0020.0002] # LATIN SMALL LETTER G WITH CEDILLA
0122      ; [.2074.0020.0008] # LATIN CAPITAL LETTER G WITH CEDILLA
0047 0327 ; [.2074.0020.0008] # LATIN CAPITAL LETTER G WITH CEDILLA
0137      ; [.20D5.0020.0002] # LATIN SMALL LETTER K WITH CEDILLA
006B 0327 ; [.20D5.0020.0002] # LATIN SMALL LETTER K WITH CEDILLA
0136      ; [.20D5.0020.0008] # LATIN CAPITAL LETTER K WITH CEDILLA
004B 0327 ; [.20D5.0020.0008] # LATIN CAPITAL LETTER K WITH CEDILLA
013C      ; [.2108.0020.0002] # LATIN SMALL LETTER L WITH CEDILLA
006C 0327 ; [.2108.0020.0002] # LATIN SMALL LETTER L WITH CEDILLA
013B      ; [.2108.0020.0008] # LATIN CAPITAL LETTER L WITH CEDILLA
004C 0327 ; [.2108.0020.0008] # LATIN CAPITAL LETTER L WITH CEDILLA
0146      ; [.213B.0020.0002] # LATIN SMALL LETTER N WITH CEDILLA
006E 0327 ; [.213B.0020.0002] # LATIN SMALL LETTER N WITH CEDILLA
0145      ; [.213B.0020.0008] # LATIN CAPITAL LETTER N WITH CEDILLA
004E 0327 ; [.213B.0020.0008] # LATIN CAPITAL LETTER N WITH CEDILLA
0157      ; [.21D1.0020.0002] # LATIN SMALL LETTER R WITH CEDILLA
0072 0327 ; [.21D1.0020.0002] # LATIN SMALL LETTER R WITH CEDILLA
0156      ; [.21D1.0020.0008] # LATIN CAPITAL LETTER R WITH CEDILLA
0052 0327 ; [.21D1.0020.0008] # LATIN CAPITAL LETTER R WITH CEDILLA
0161      ; [.21F6.0020.0002] # LATIN SMALL LETTER S WITH CARON
0073 030C ; [.21F6.0020.0002] # LATIN SMALL LETTER S WITH CARON
0160      ; [.21F6.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
0053 030C ; [.21F6.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
017E      ; [.22A2.0020.0002] # LATIN SMALL LETTER Z WITH CARON
007A 030C ; [.22A2.0020.0002] # LATIN SMALL LETTER Z WITH CARON
017D      ; [.22A2.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
005A 030C ; [.22A2.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
ENTRY
};
