+{
   locale_version => 1.29,
   entry => <<'ENTRY', # for DUCET v13.0.0
0D3D      ; [.0000.00C6.0002] # MALAYALAM SIGN AVAGRAHA
0D57      ; [.2C4D.0020.0002] # MALAYALAM AU LENGTH MARK
0D4C      ; [.2C4D.0020.0003] # MALAYALAM VOWEL SIGN AU
0D46 0D57 ; [.2C4D.0020.0003] # MALAYALAM VOWEL SIGN AU
0D15 0D4D 200D ; [.2C17.0020.0002][.2C4F.0021.0002] # <MALAYALAM LETTER KA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7F      ; [.2C17.0020.0003][.2C4F.0021.0002] # MALAYALAM LETTER CHILLU K
0D23 0D4D 200D ; [.2C25.0020.0002][.2C4F.0021.0002] # <MALAYALAM LETTER NNA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7A      ; [.2C25.0020.0003][.2C4F.0021.0002] # MALAYALAM LETTER CHILLU NN
0D28 0D4D 200D ; [.2C2A.0020.0002][.2C4F.0021.0002] # <MALAYALAM LETTER NA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7B      ; [.2C2A.0020.0003][.2C4F.0021.0002] # MALAYALAM LETTER CHILLU N
0D30 0D4D 200D ; [.2C32.0020.0002][.2C4F.0021.0002] # <MALAYALAM LETTER RA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7C      ; [.2C32.0020.0003][.2C4F.0021.0002] # MALAYALAM LETTER CHILLU RR
0D32 0D4D 200D ; [.2C33.0020.0002][.2C4F.0021.0002] # <MALAYALAM LETTER LA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7D      ; [.2C33.0020.0003][.2C4F.0021.0002] # MALAYALAM LETTER CHILLU L
0D33 0D4D 200D ; [.2C39.0020.0002][.2C4F.0021.0002] # <MALAYALAM LETTER LLA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7E      ; [.2C39.0020.0003][.2C4F.0021.0002] # MALAYALAM LETTER CHILLU LL
0D02      ; [.2C30.0020.0002][.2C4F.0021.0002] # MALAYALAM SIGN ANUSVARA
0D7B 0D4D ; [.2C2A.0020.0002][.2C4F.0020.0003] # <MALAYALAM LETTER CHILLU N, MALAYALAM SIGN VIRAMA>
ENTRY
};
